import numpy as np
import pandas as pd
from bs4 import BeautifulSoup
import copy

def getDataframe(table, raw=False, strip=True):
    """extract pandas dataframe from a well-formatted HTML table on Wikipedia with rowspans/colspans attributes in <td> tag.
    Parameters:
       table (str): html table
       raw (bool):  get inner html; or get text
       strip (bool): strip whitespaces, when raw == False
    Returns:
       pandas: dataframe extracted from html table
    """

    trs = table.findAll('tr')

    ##1. assumes top row is header
    header= []
    children = trs[0].findChildren(recursive=False)
    for child in children:
        header.append(child.find(text= True).strip())

    ##2. create dataframe to capture table values
    df_table_values = pd.DataFrame(index=np.arange(len(trs)), columns=header)
    df_table_values = df_table_values.replace({np.nan: None})

    ##3. prepare tds that have both rowspan and colspan
    for i in range(1, len(trs)):
        children= trs[i].findChildren('td', recursive=False)
        for j in range(0, len(children)):
            cspan = int(children[j].get('colspan')) if children[j].get('colspan') else 1
            rspan = int(children[j].get('rowspan')) if children[j].get('rowspan') else 1
            if cspan > 1 and rspan > 1:
                del children[j]['colspan']
                for c in range(1, cspan):
                    td_copy = copy.copy(children[j])
                    children[j].insert_after(td_copy)

    ##4. assign cell values bases on table spans
    for i in range(1, len(trs)):
        val_row = i
        children= trs[i].findChildren('td', recursive=False)
        for j in range(0, len(children)):
            ##what value to assign to cell(s)
            if raw:
                val = children[j].decode_contents()
            else:
                if strip:
                    val = children[j].find(text=True).strip()
                else:
                    val = children[j].find(text=True)

            val_col = j
            while True:
                if df_table_values.iloc[val_row, val_col] is None:
                    break
                val_col= val_col + 1
            df_table_values.iloc[val_row, val_col]= val

            cspan = int(children[j].get('colspan')) if children[j].get('colspan') else 1
            rspan = int(children[j].get('rowspan')) if children[j].get('rowspan') else 1

            ##handle when colspan alone is present
            if cspan > 1 and rspan == 1:
                for c in range(val_col+1, val_col+cspan):
                    df_table_values.iloc[val_row, c] = val

            ##handle when rowspan alone is present
            if rspan > 1 and cspan == 1:
                c= val_col
                for r in range(val_row+1, val_row+rspan):
                    while True:
                        if df_table_values.iloc[r, c] is None:
                            break
                        c = c + 1
                    df_table_values.iloc[r, c] = val


    ##4. drop first row
    df_table_values= df_table_values.iloc[1:, :]
    df_table_values.reset_index(inplace= True)

    return df_table_values


